# Copyright 2023 Le Filament (https://le-filament.com)
# License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl.html)

from odoo import http
from odoo.http import request

from odoo.addons.portal.controllers.portal import CustomerPortal


class PortalAccount(CustomerPortal):
    # ------------------------------------------------------
    # Inherit parent
    # ------------------------------------------------------
    def _prepare_portal_layout_values(self):
        """
        Hérite la page d'accueil pour afficher le lien vers les dossiers partagés
        """
        values = super()._prepare_portal_layout_values()
        hall_ids = self._get_hall_ids()
        contract_ids = self._get_contract_ids()
        is_discount = True if hall_ids.filtered("is_hall_discount") else False
        if hall_ids and hall_ids.mapped("expenses_link"):
            values["hall_ids"] = hall_ids
            values["contract_ids"] = contract_ids
            values["is_discount"] = is_discount
        return values

    def _prepare_home_portal_values(self, counters):
        """
        Hérite les compteurs pour l'affichage des loyers vertueux
        """
        values = super()._prepare_home_portal_values(counters)
        if "discount_count" in counters:
            discount_count = len(self._get_contract_ids().discount_ids)
            values["discount_count"] = discount_count
        return values

    # ------------------------------------------------------
    # New routes
    # ------------------------------------------------------
    @http.route(
        ["/my/discounts/<int:contract_id>"], type="http", auth="user", website=True
    )
    def portal_my_discounts(self, contract_id, **kw):
        contract_id = request.env["hall.contract"].browse(contract_id)
        discount_ids = contract_id.discount_ids
        values = {
            "contract_id": contract_id,
            "new_disc_ids": discount_ids.filtered(lambda d: d.state == "to_submit"),
            "submit_disc_ids": discount_ids.filtered(lambda d: d.state != "to_submit"),
            "title": "Loyers vertueux | Festa",
        }
        return request.render("festa_portal.portal_my_discounts", values)

    @http.route(
        ["/my/discount/<int:discount_id>/edit"],
        type="http",
        auth="user",
        website=True,
        methods=["GET"],
    )
    def portal_discount_edit(self, discount_id):
        disc_id = request.env["hall.contract.discount"].browse(discount_id)
        values = {
            "disc_id": disc_id,
            "title": "Loyer vertueux mensuel | Festa",
        }
        return request.render("festa_portal.portal_discount_edit", values)

    @http.route(
        ["/my/discount/<int:discount_id>/save"],
        type="http",
        auth="user",
        website=True,
        methods=["POST"],
    )
    def portal_discount_post(self, discount_id, **post):
        disc_id = request.env["hall.contract.discount"].browse(discount_id)
        checked_ids = [
            int(q_id) for q_id in request.httprequest.form.getlist("is_valid")
        ]
        all_question_ids = disc_id.criteria_ids.question_ids
        to_check = list(set(checked_ids) & set(all_question_ids.ids))
        to_uncheck = list(set(all_question_ids.ids) - set(checked_ids))
        all_question_ids.browse(to_check).update({"is_valid": True})
        all_question_ids.browse(to_uncheck).update({"is_valid": False})
        for i in all_question_ids.ids:
            if post.get("customer_comment_%d" % i):
                all_question_ids.browse(i).update(
                    {"customer_comment": post.get("customer_comment_%d" % i)}
                )
        if post.get("action") == "save":
            return request.redirect("my/discounts/%d" % disc_id.contract_id.id)
        if post.get("action") == "send":
            for criteria in disc_id.criteria_ids.sudo():
                criteria.update({"state": "to_validate"})
            if disc_id.sudo().contract_id.hall_id.discount_user_ids:
                template = request.env.ref(
                    "festa_portal.email_template_discount_validate"
                ).sudo()
                template.send_mail(
                    res_id=disc_id.id,
                    force_send=True,
                    email_layout_xmlid="mail.mail_notification_light",
                )
            return request.redirect("my/discounts/%d" % disc_id.contract_id.id)

    # ------------------------------------------------------
    # Business methods
    # ------------------------------------------------------
    def _get_contract_ids(self):
        contract_ids = request.env["hall.contract"].search(
            [("partner_id", "=", request.env.user.commercial_partner_id.id)]
        )
        return contract_ids

    def _get_hall_ids(self):
        return self._get_contract_ids().sudo().mapped("hall_id")
